@@ -704,11 +704,11 @@ typedef enum {
 	_GMATHML_STROKE_WIDTH_ODD,
 	_GMATHML_STROKE_WIDTH_NULL,
 	_GMATHML_STROKE_WIDTH_VECTOR
-} _GMathmlStrokeWidth;
+} _LsmMathmlStrokeWidth;
 
 static void
 _round_rectangle_coordinates (cairo_t *cairo,
-			      _GMathmlStrokeWidth stroke_width,
+			      _LsmMathmlStrokeWidth stroke_width,
 			      double *x0, double *y0,
 			      double *x1, double *y1)
 {
@@ -733,11 +733,11 @@ _round_rectangle_coordinates (cairo_t *cairo,
 	}
 }
 
-static _GMathmlStrokeWidth
+static _LsmMathmlStrokeWidth
 _emit_stroke_attributes (LsmMathmlView *view, LsmMathmlLine line, double line_width,
 			 const LsmMathmlColor *color)
 {
-	_GMathmlStrokeWidth stroke_width;
+	_LsmMathmlStrokeWidth stroke_width;
 	cairo_t *cairo;
 	double dashes[2] = {3.0, 2.0};
 	double rounded_width = line_width;
@@ -848,7 +848,7 @@ lsm_mathml_view_show_rectangle (LsmMathmlView *view,
 			     double x0, double y0, double width, double height,
 			     LsmMathmlLine line, double line_width)
 {
-	_GMathmlStrokeWidth stroke_width;
+	_LsmMathmlStrokeWidth stroke_width;
 	cairo_t *cairo;
 	double x1, y1;
 
@@ -877,7 +877,7 @@ lsm_mathml_view_show_line (LsmMathmlView *view,
 			double x0, double y0, double x1, double y1,
 			LsmMathmlLine line, double line_width)
 {
-	_GMathmlStrokeWidth stroke_width;
+	_LsmMathmlStrokeWidth stroke_width;
 	cairo_t *cairo;
 
 	g_return_if_fail (LSM_IS_MATHML_VIEW (view));
@@ -903,7 +903,7 @@ lsm_mathml_view_show_fraction_line (LsmMathmlView *view,
 				 double x, double y,
 				 double width, double thickness)
 {
-	_GMathmlStrokeWidth stroke_width;
+	_LsmMathmlStrokeWidth stroke_width;
 	cairo_t *cairo;
 	double x0, y0, x1, y1;
 
@@ -960,32 +960,33 @@ lsm_mathml_view_measure (LsmDomView *dom_view, double *width, double *height)
 }
 
 static void
-lsm_mathml_view_render (LsmDomView *dom_view, double x, double y)
+lsm_mathml_view_render (LsmDomView *dom_view)
 {
 	LsmMathmlView *view = LSM_MATHML_VIEW (dom_view);
 	LsmMathmlMathElement *math_element;
 	cairo_t *cairo;
 	const LsmMathmlBbox *bbox;
+	double resolution_ppi;
 
 	math_element = lsm_mathml_document_get_root_element (LSM_MATHML_DOCUMENT (view->dom_view.document));
 	if (math_element == NULL)
 		return;
 
+	resolution_ppi = lsm_dom_document_get_resolution (view->dom_view.document);
+
 	bbox = _view_measure (view, NULL, NULL);
 
 	lsm_mathml_math_element_layout (math_element, view, bbox);
 
 	cairo = view->dom_view.cairo;
 
-	cairo_save (view->dom_view.cairo);
+	cairo_scale (cairo, resolution_ppi / 72.0, resolution_ppi / 72.0);
 
-	cairo_translate (cairo, x, y + bbox->height);
+	cairo_translate (cairo, 0, bbox->height);
 
 	lsm_mathml_math_element_render (math_element, view);
 
 	lsm_debug ("[LsmMathmlView::render] cairo status = %s", cairo_status_to_string (cairo_status (cairo)));
-
-	cairo_restore (cairo);
 }
 
 LsmMathmlView *
