@@ -26,6 +26,11 @@
 
 G_BEGIN_DECLS
 
+typedef enum {
+	LSM_SVG_ANGLE_TYPE_AUTO,
+	LSM_SVG_ANGLE_TYPE_FIXED
+} LsmSvgAngleType;
+
 typedef enum {
 	LSM_SVG_PAINT_TYPE_UNKNOWN = 0,
 	LSM_SVG_PAINT_TYPE_RGB_COLOR,
@@ -106,6 +111,14 @@ typedef enum {
 const char * 		lsm_svg_pattern_units_to_string 		(LsmSvgPatternUnits units);
 LsmSvgPatternUnits	lsm_svg_pattern_units_from_string		(const char *string);
 
+typedef enum {
+	LSM_SVG_MARKER_UNITS_USER_SPACE_ON_USE,
+	LSM_SVG_MARKER_UNITS_STROKE_WIDTH
+} LsmSvgMarkerUnits;
+
+const char * 		lsm_svg_marker_units_to_string 			(LsmSvgMarkerUnits units);
+LsmSvgMarkerUnits	lsm_svg_marker_units_from_string		(const char *string);
+
 typedef enum {
 	LSM_SVG_SPREAD_METHOD_PAD,
 	LSM_SVG_SPREAD_METHOD_REFLECT,
@@ -115,6 +128,32 @@ typedef enum {
 const char * 		lsm_svg_spread_method_to_string 		(LsmSvgSpreadMethod method);
 LsmSvgSpreadMethod	lsm_svg_spread_method_from_string		(const char *string);
 
+typedef enum {
+	LSM_SVG_ALIGN_UNKNOWN,
+	LSM_SVG_ALIGN_NONE,
+	LSM_SVG_ALIGN_X_MIN_Y_MIN,
+	LSM_SVG_ALIGN_X_MID_Y_MIN,
+	LSM_SVG_ALIGN_X_MAX_Y_MIN,
+	LSM_SVG_ALIGN_X_MIN_Y_MID,
+	LSM_SVG_ALIGN_X_MID_Y_MID,
+	LSM_SVG_ALIGN_X_MAX_Y_MID,
+	LSM_SVG_ALIGN_X_MIN_Y_MAX,
+	LSM_SVG_ALIGN_X_MID_Y_MAX,
+	LSM_SVG_ALIGN_X_MAX_Y_MAX
+} LsmSvgAlign;
+
+const char * 		lsm_svg_align_to_string 		(LsmSvgAlign align);
+LsmSvgAlign		lsm_svg_align_from_string		(const char *string);
+
+typedef enum {
+	LSM_SVG_MEET_OR_SLICE_UNKNOWN,
+	LSM_SVG_MEET_OR_SLICE_MEET,
+	LSM_SVG_MEET_OR_SLICE_SLICE
+} LsmSvgMeetOrSlice;
+
+const char * 		lsm_svg_meet_or_slice_to_string 	(LsmSvgMeetOrSlice meet_or_slice);
+LsmSvgMeetOrSlice	lsm_svg_meet_or_slice_from_string	(const char *string);
+
 G_END_DECLS
 
 #endif
